import os
import socketserver

from shoot.server import ShootServer


def run():
    handler = ShootServer
    httpd = socketserver.TCPServer(("", 8000), handler)
    print("serving at port 8000...")
    httpd.serve_forever()



import os
import sys

from django.template import Context
import django.template.loader

CWD = os.getcwd()
sys.path.append(CWD)

if os.path.exists(os.path.join(CWD, 'django_settings.py')):
    django_settings_module = 'django_settings'
    print("found custom django settings...")
else:
    django_settings_module = 'shoot.django_settings'
    print('no custom django settings found, using defaults..')
os.environ.setdefault("DJANGO_SETTINGS_MODULE", django_settings_module)

config = {}

try:
    import shootconfig
    print("Custom context found...")
    config['context'] = Context(shootconfig.context)
except ImportError:
    print("No custom context found, using defaults...")
    config['context'] = Context()



import os
import io
import http.server
from django.template import Template

from shoot import config


class ShootServer(http.server.SimpleHTTPRequestHandler):
    """Handler to deal with incoming requests.
    """

    def gen_response_stream(self, f):
        template = Template(f.read())
        content = template.render(config.get('context'))
        content_in_bytes = bytes(content, 'UTF-8')
        stream = io.BytesIO(content_in_bytes)
        return (stream, len(content_in_bytes))

    def send_head(self):
        """Builds response headers and in process renders templates, if any.

        This method overrides SimpleHTTPRequestHandlet.send_head()
        """
        path = self.translate_path(self.path)
        f = None
        to_render = False
        if os.path.isdir(path):
            if not self.path.endswith('/'):
                # redirect browser - doing basically what apache does
                self.send_response(301)
                self.send_header("Location", self.path + "/")
                self.end_headers()
                return None
        else:
            # check if URL corresponds to a template to render
            if path.endswith("/"):
                index = path[:-1]
            else:
                index = path
            for ext in '.html', '.htm':
                if os.path.exists(index + ext):
                    to_render = True
                    realpath = index + ext
                    break
        if os.path.isdir(path):
            # if dir, check for existence of index.htm*
            for index in "index.html", "index.htm":
                index = os.path.join(path, index)
                if os.path.exists(index):
                    realpath = index
                    to_render = True
                    break
            else:
                return self.list_directory(path)
        # deny if URL directly requests for *.html file, allow if dir
        file_extension = os.path.splitext(path)[1]
        if file_extension in ('.html', '.htm') and not os.path.isdir(path):
            self.send_error(404, "File not found")
            return None
        if to_render:
            path = realpath
        ctype = self.guess_type(path)
        try:
            f = open(path, 'rb')
        except IOError:
            self.send_error(404, "File not found")
            return None
        fs = os.fstat(f.fileno())
        if to_render:
            stream, length = self.gen_response_stream(f)
        else:
            length = fs[6]
        self.send_response(200)
        self.send_header("Content-type", ctype)
        self.send_header("Content-Length", str(length))
        self.send_header("Last-Modified", self.date_time_string(
            fs.st_mtime))
        self.end_headers()
        if to_render:
            return stream
        return f



import setuptools


setuptools.setup(name='Shoot',
                 version='0.1.1',
                 author='Phaneendra Chiruvella',
                 author_email='hi@pcx.io',
                 packages=['shoot',],
                 url='https://github.com/pcx/shoot',
                 description='Kick-start your prototype with Django templates',
                 install_requires=["Django",],
                 entry_points={
                     'console_scripts': [
                         ['shoot = shoot.run:run',],
                     ]
                 },
                 package_data={'': ['LICENSE.txt', 'README.rst']},
                 include_package_data=True,
                 license=open('LICENSE').read(),
                 long_description=open('README.rst').read(),
             )
